import{h as a}from"./axios--FnjuHWf.js";import{d as n,r as c,o as r,b as i,c as _,e as o,h as s,t as d}from"./index-eJPIzJhh.js";const l={style:{"background-color":"#d2d2fa",height:"100vh",display:"grid","place-items":"center"}},p={style:{"text-align":"center"}},h=o("a",{href:"https://github.com/xaoyaoo/PyWxDump.git"},"PyWxDump",-1),m={style:{"text-align":"center"}},v=n({__name:"HomeView",setup(g){const t=c("");return r(async()=>{try{const e=await a.get("/api/version");t.value=e}catch(e){t.value="获取版本失败",console.log(e)}}),(e,y)=>(i(),_("div",l,[o("h2",p,[s("欢迎使用"),h,s("聊天记录查看工具! "),o("p",m,"当前版本："+d(t.value),1)])]))}});export{v as _};
