.label_color[data-v-0c9eea5d]{color:#333;font-size:15px;padding-left:15px;padding-right:0}.data_color[data-v-0c9eea5d]{color:#08488c;background-color:#f4f4f4;font-size:15px;padding-left:6px;padding-right:6px;font-weight:700;white-space:nowrap;max-width:80%}.button_color[data-v-0c9eea5d]{color:#0048ff;font-size:15px;padding-left:15px;padding-right:0;text-decoration:underline}.block[data-v-320582f6]{padding:30px 0;text-align:center;border-right:solid 1px var(--el-border-color);flex:1}.block[data-v-320582f6]:last-child{border-right:none}.block .demonstration[data-v-320582f6]{display:block;color:var(--el-text-color-secondary);font-size:14px;margin-bottom:20px}
